@@ -4,23 +4,21 @@
 # LICENSE file in the root directory.
 
 """ Numpy based implementation of multi objective
-    Differential Evolution using the DE/pareto/1 strategy. 
-    Derived and adapted for MO from its C++ counterpart 
-    https://github.com/dietmarwo/fast-cma-es/blob/master/_fcmaescpp/deoptimizer.cpp
-    
+    Differential Evolution using either DE/rand/1 or DE/best/1 strategy ('best' refers to the current pareto front'). 
+     
     Can switch to NSGA-II like population update via parameter 'nsga_update'.
     Then it works essentially like NSGA-II but instead of the tournament selection
     the whole population is sorted and the best individuals survive. To do this
-    efficiently the crowd distance ordering is slightly inaccurate - each objective
-    is ordered separately. 
+    efficiently the crowd distance ordering is slightly inaccurate. 
     
-    Compared to most Python NSGA-II implementations it supports parallel fitness function 
-    evaluation. 
+    Supports parallel fitness function evaluation. 
     
+    Features enhanced multiple constraint ranking (https://www.jstage.jst.go.jp/article/tjpnsec/11/2/11_18/_article/-char/en/)
+    improving its performance in handling constraints for engineering design optimization.
+
     Enables the comparison of DE and NSGA-II population update mechanism with everything else
     kept completely identical.
     
-    
     Requires python 3.5 or higher. 
     
     Uses the following deviation from the standard DE algorithm:
@@ -30,26 +28,27 @@
     since the algorithm oscillates between different F and CR settings. 
     
     For expensive objective functions (e.g. machine learning parameter optimization) use the workers
-    parameter to parallelize objective function evaluation. This causes delayed population update.
-    It is usually preferrable if popsize > workers and workers = mp.cpu_count() to improve CPU utilization.  
+    parameter to parallelize objective function evaluation. The workers parameter is limited by the 
+    population size.
 """
 
 import numpy as np
+import os
 import sys
 import time
 import ctypes as ct
 from numpy.random import Generator, MT19937
-from scipy.optimize import OptimizeResult
 from fcmaes.evaluator import Evaluator
 import multiprocessing as mp
-from fcmaes.optimizer import dtime, logger
-from fcmaes import retry, moretry
+from fcmaes.optimizer import logger
+from fcmaes import moretry
+
+os.environ['MKL_DEBUG_CPU_TYPE'] = '5'
 
 def minimize(mofun, 
              nobj,
-             dim = None,
-             bounds = None,
-             ncon = 0, 
+             ncon, 
+             bounds,
              popsize = 64, 
              max_evaluations = 100000, 
              workers = None,
@@ -66,25 +65,22 @@ def minimize(mofun,
      
     Parameters
     ----------
-    fun : callable
+    mofun : callable
         The objective function to be minimized.
-            ``fun(x, *args) -> float``
+            ``mofun(x, *args) -> list(float)``
         where ``x`` is an 1-D array with shape (n,) and ``args``
         is a tuple of the fixed parameters needed to completely
         specify the function.
     nobj : int
         number of objectives
-    dim : int
-        dimension of the argument of the objective function
-        either dim or bounds need to be defined
-    bounds : sequence or `Bounds`, optional
+    ncon : int
+        number of constraints, default is 0. 
+        The objective function needs to return vectors of size nobj + ncon
+    bounds : sequence or `Bounds`
         Bounds on variables. There are two ways to specify the bounds:
             1. Instance of the `scipy.Bounds` class.
             2. Sequence of ``(min, max)`` pairs for each element in `x`. None
                is used to specify no bound.
-    ncon : int, optional
-        number of constraints, default is 0. 
-        The objective function needs to return vectors of size nobj + ncon
     popsize : int, optional
         Population size.
     max_evaluations : int, optional
@@ -108,40 +104,32 @@ def minimize(mofun,
         logger for log output for tell_one, If None, logging
         is switched off. Default is a logger which logs both to stdout and
         appends to a file ``optimizer.log``.
-    trace_plots : trace_plots, optional
-        flag indicating if the pareto front is plotted during the optimization to monitor progress
+    plot_name : plot_name, optional
+        if defined the pareto front is plotted during the optimization to monitor progress
             
     Returns
     -------
-    res : scipy.OptimizeResult
-        The optimization result is represented as an ``OptimizeResult`` object.
-        Important attributes are: ``x`` the solution array, 
-        ``fun`` the best function value, 
-        ``nfev`` the number of function evaluations,
-        ``nit`` the number of iterations,
-        ``success`` a Boolean flag indicating if the optimizer exited successfully. """
+    x, y: list of argument vectors and corresponding value vectors of the optimization results. """
 
-    
-    de = DE(dim, nobj, bounds, ncon, popsize, workers if not workers is None else 0, 
+    mode = MODE(nobj, ncon, bounds, popsize, workers if not workers is None else 0, 
             f, cr, nsga_update, pareto_update, rg, logger, plot_name)
     try:
         if workers and workers > 1:
-            x, y, evals, iterations, stop = de.do_optimize_delayed_update(mofun, max_evaluations, workers)
+            x, y, evals, iterations, stop = mode.do_optimize_delayed_update(mofun, max_evaluations, workers)
         else:      
-            x, y, evals, iterations, stop = de.do_optimize(mofun, max_evaluations)
-        return OptimizeResult(x=x, fun=y, nfev=evals, nit=iterations, status=stop, 
-                              success=True)
+            x, y, evals, iterations, stop = mode.do_optimize(mofun, max_evaluations)
+        return x, y
     except Exception as ex:
-        return OptimizeResult(x=None, fun=sys.float_info.max, nfev=0, nit=0, status=-1, success=False)  
+        return None, None  
 
-class DE(object):
+class MODE(object):
     
-    def __init__(self, dim, nobj, bounds, ncon=0, popsize = 64, workers = 0,
+    def __init__(self, nobj, ncon, bounds, popsize = 64, workers = 0,
                  F = 0.5, Cr = 0.9, nsga_update = False, pareto_update = False, 
                  rg = Generator(MT19937()), logger = None, plot_name = None):
         self.nobj = nobj
         self.ncon = ncon
-        self.dim, self.lower, self.upper = _check_bounds(bounds, dim)
+        self.dim, self.lower, self.upper = _check_bounds(bounds, None)
         if popsize is None:
             popsize = 31
         self.popsize = popsize
@@ -157,7 +145,6 @@ class DE(object):
         self.mutex = mp.Lock()
         self.p = 0
         self.best_p = None
-        self.trace_plots = not plot_name is None
         self._init()
         if not logger is None:
             self.logger = logger
@@ -221,7 +208,7 @@ class DE(object):
                 message = '"c/t={0:.2f} c={1:d} t={2:.2f} y={3!s} x={4!s}'.format(
                     c/t, c, t, str(list(y)), str(list(x)))
                 self.logger.info(message)   
-                if self.trace_plots:             
+                if hasattr(self, "plot_name") and not self.plot_name is None:           
                     name = self.name + ' '  + str(self.n_evals.value)
                     np.savez_compressed(name, xs=self.x, ys=self.y) 
                     moretry.plot(name, self.ncon, self.x, self.y)
@@ -254,8 +241,6 @@ class DE(object):
                 self.x[self.popsize + p] = x
                 self.evals += 1
             self.pop_update()
-            if self.evals >= self.max_evals:
-                break
         x, y = filter(self.x, self.y)
         return x, y, self.evals, self.iterations, self.stop
 
@@ -376,39 +361,57 @@ def _check_bounds(bounds, dim):
         return len(bounds.ub), np.asarray(bounds.lb), np.asarray(bounds.ub)
  
 def filter(x, y):
-    sorted = np.argsort([yi[0] for yi in y])
+    ym = np.amax(y,axis=1)
+    sorted = np.argsort(ym)
     x = x[sorted]
     y = y[sorted]
     y = np.array([yi for yi in y if yi[0] < 1E99])
     x = np.array(x[:len(y)])
     return x,y
 
-def maxcon(cons):
-    cons = np.minimum(cons, 1E99)
-    maxc = max(cons)
-    if maxc == 0:
-        return 0
-    else: # reduce number and value of constraint violations
-        n = sum([c > 0 for c in cons])
-        return n*maxc + sum(cons)
+def objranks(objs):
+    ci = objs.argsort(axis=0)
+    rank = np.empty_like(ci)
+    ar = np.arange(objs.shape[0])
+    for i in range(objs.shape[1]): 
+        rank[ci[:,i], i] = ar 
+    rank = np.sum(rank, axis=1)
+    return rank
 
+def ranks(cons):
+    feasible = np.less_equal(cons, 0)
+    ci = cons.argsort(axis=0)
+    rank = np.empty_like(ci)
+    ar = np.arange(cons.shape[0])
+    for i in range(cons.shape[1]): 
+        rank[ci[:,i], i] = ar
+    rank[feasible] = 0
+    alpha = np.sum(np.greater(cons, 0), axis=1) / cons.shape[1] # violations
+    alpha = np.tile(alpha, (cons.shape[1],1)).T
+    rank = rank*alpha
+    rank = np.sum(rank, axis=1)
+    return rank
+     
 def pareto(ys, nobj, ncon):
     if ncon == 0:
         return pareto_levels(ys)
     else:
         yobj = np.array([y[:nobj] for y in ys])
         ycon = np.array([np.maximum(y[-ncon:], 0) for y in ys])    
-        csum = [maxcon(cons) for cons in ycon]
+        csum = ranks(ycon)
+        feasible = np.less_equal(csum, 0)
+        if sum(feasible) > 0:
+            csum += objranks(yobj)
         ci = np.argsort(csum)
         popn = len(ys)
         domination = np.zeros(popn)
         # first pareto front of feasible solutions
-        cy = np.array([i for i in ci if csum[i] == 0])
+        cy = np.array([i for i in ci if feasible[i]])
         if len(cy) > 0:
             ypar = pareto_levels(yobj[cy])
-            domination[cy] += ypar
+            domination[cy] += ypar        
         # then constraint violations   
-        ci = np.array([i for i in ci if csum[i] > 0 and csum[i] < 1E99])  
+        ci = np.array([i for i in ci if not feasible[i]])  
         if len(ci) > 0:    
             maxcdom = len(ci)
             cdom = np.arange(maxcdom, 0, -1)
@@ -461,7 +464,8 @@ def variation(pop, lower, upper, rg, pro_c = 1, dis_c = 20, pro_m = 1, dis_m = 2
     beta[mu > 0.5] = np.power(2 * mu[mu > 0.5], -1 / (dis_c + 1))
     beta = beta * ((-1)** rg.integers(2, size=(n // 2, d)))
     beta[rg.random((n // 2, d)) < 0.5] = 1
-    beta[np.tile(rg.random((n // 2, 1)) > pro_c, (1, d))] = 1
+    if pro_c < 1.0:
+        beta[np.tile(rg.random((n // 2, 1)) > pro_c, (1, d))] = 1
     offspring = np.vstack(((parent_1 + parent_2) / 2 + beta * (parent_1 - parent_2) / 2,
                                (parent_1 + parent_2) / 2 - beta * (parent_1 - parent_2) / 2))
     site = rg.random((n, d)) < pro_m / d
@@ -481,15 +485,13 @@ def variation(pop, lower, upper, rg, pro_c = 1, dis_c = 20, pro_m = 1, dis_m = 2
     offspring = np.maximum(np.minimum(offspring, upper), lower)
     return offspring
 
-def minimize_plot(name, fun, bounds, nobj, ncon = 0, popsize = 64, max_eval = 100000, nsga_update=False, 
+def minimize_plot(name, fun, nobj, ncon, bounds, popsize = 64, max_evaluations = 100000, nsga_update=False, 
                   pareto_update=False, workers = mp.cpu_count(), logger=logger(), plot_name = None):
     name += '_mode_' + str(popsize) + '_' + \
                 ('nsga_update' if nsga_update else ('de_best_update' if pareto_update else 'de_all_update'))
     logger.info('optimize ' + name) 
-    ret = minimize(fun, nobj, bounds = bounds, ncon = ncon, popsize = popsize, max_evaluations = max_eval,
+    xs, ys = minimize(fun, nobj, ncon, bounds, popsize = popsize, max_evaluations = max_evaluations,
                    nsga_update = nsga_update, pareto_update = pareto_update, workers=workers, 
                    logger=logger, plot_name = plot_name)
-    ys = np.array(ret.fun)
-    xs = np.array(ret.x)
     np.savez_compressed(name, xs=xs, ys=ys)
     moretry.plot(name, ncon, xs, ys)
